package main

import (
	"context"
	"fmt"
	"os"
	"os/signal"
	"syscall"
	"time"

	"k8s.io/client-go/kubernetes"
	"k8s.io/client-go/rest"
	"libre.sh/keydb-manager/internal"
)

func main() {
	config := internal.Config{
		Namespace: os.Getenv("NAMESPACE"),
		Instance:  os.Getenv("INSTANCE"),
		Hostname:  os.Getenv("HOSTNAME"),
		Pass:      os.Getenv("PASS"),
	}

	kubconfig, err := rest.InClusterConfig()
	if err != nil {
		panic(err.Error())
	}
	clientset, err := kubernetes.NewForConfig(kubconfig)
	if err != nil {
		panic(err.Error())
	}

	ctx, cancel1 := signal.NotifyContext(context.Background(), os.Interrupt, syscall.SIGTERM, syscall.SIGINT)
	defer cancel1()

	manager := internal.NewManager(config)
	informer := internal.NewInformer(ctx, config, clientset, manager)
	server := internal.NewServer(ctx, config, clientset, manager)

	go informer.Run(ctx.Done())
	go server.ListenAndServe()

	<-ctx.Done()

	ctx, cancel2 := context.WithTimeout(context.Background(), time.Second*5)
	defer cancel2()
	if err := server.Shutdown(ctx); err != nil {
		fmt.Println(err)
	}
}
